@@ -87,19 +87,10 @@ void QLibrarySettings::load()
     }
 }
 
-namespace {
-const QString *qtconfManualPath = nullptr;
-}
-
-void QLibraryInfoPrivate::setQtconfManualPath(const QString *path)
-{
-    qtconfManualPath = path;
-}
-
 static QSettings *findConfiguration()
 {
-    if (qtconfManualPath)
-        return new QSettings(*qtconfManualPath, QSettings::IniFormat);
+    if (QLibraryInfoPrivate::qtconfManualPath)
+        return new QSettings(*QLibraryInfoPrivate::qtconfManualPath, QSettings::IniFormat);
 
     QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
     if (QFile::exists(qtconfig))
@@ -131,6 +122,8 @@ static QSettings *findConfiguration()
     return nullptr;     //no luck
 }
 
+const QString *QLibraryInfoPrivate::qtconfManualPath = nullptr;
+
 QSettings *QLibraryInfoPrivate::configuration()
 {
     QLibrarySettings *ls = qt_library_settings();
@@ -482,7 +475,6 @@ QLibraryInfoPrivate::LocationInfo QLibraryInfoPrivate::locationInfo(QLibraryInfo
         "Examples", "examples",
         "Tests", "tests"
     );
-    [[maybe_unused]]
     constexpr QByteArrayView dot{"."};
 
     LocationInfo result;
@@ -751,14 +743,13 @@ extern "C" void qt_core_boilerplate() __attribute__((force_align_arg_pointer));
 void qt_core_boilerplate()
 {
     printf("This is the QtCore library version %s\n"
-           "%s\n"
-           "Contact: https://www.qt.io/licensing/\n"
+           "Copyright (C) 2023 The Qt Company Ltd.\n"
+           "Contact: http://www.qt.io/licensing/\n"
            "\n"
            "Installation prefix: %s\n"
            "Library path:        %s\n"
            "Plugin path:         %s\n",
            QT_PREPEND_NAMESPACE(qt_build_string)(),
-           QT_COPYRIGHT,
            QT_CONFIGURE_PREFIX_PATH,
            qt_configure_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::LibrariesPath - 1],
            qt_configure_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::PluginsPath - 1]);
