@@ -72,7 +72,8 @@ public:
     uint isQuickItem : 1;
     uint willBeWidget : 1; // for handling widget-specific bits in QObject's ctor
     uint wasWidget : 1; // for properly cleaning up in QObject's dtor
-    uint unused : 21;
+    uint receiveParentEvents: 1;
+    uint unused : 20;
     QAtomicInt postedEvents;
     QDynamicMetaObjectData *metaObject;
     QBindingStorage bindingStorage;
@@ -235,6 +236,7 @@ public:
                            type, types, &SignalType::Object::staticMetaObject);
     }
 
+#ifndef QT_NO_CONTEXTLESS_CONNECT
     //connect without context
     template <typename Func1, typename Func2>
     static inline QMetaObject::Connection
@@ -242,6 +244,7 @@ public:
     {
         return connect(sender, signal, sender, std::forward<Func2>(slot), Qt::DirectConnection);
     }
+#endif // QT_NO_CONTEXTLESS_CONNECT
 #endif //Q_QDOC
 
     static bool disconnect(const QObject *sender, const char *signal,
@@ -351,7 +354,6 @@ private:
     bool doSetProperty(const char *name, const QVariant *lvalue, QVariant *rvalue);
 
     Q_DISABLE_COPY(QObject)
-    Q_PRIVATE_SLOT(d_func(), void _q_reregisterTimers(void *))
 
 private:
     static QMetaObject::Connection connectImpl(const QObject *sender, void **signal,
@@ -382,6 +384,8 @@ bool QObject::setProperty(const char *name, QVariant &&value)
 template <class T>
 inline T qobject_cast(QObject *object)
 {
+    static_assert(std::is_pointer_v<T>,
+                  "qobject_cast requires to cast towards a pointer type");
     typedef typename std::remove_cv<typename std::remove_pointer<T>::type>::type ObjType;
     static_assert(QtPrivate::HasQ_OBJECT_Macro<ObjType>::Value,
                     "qobject_cast requires the type to have a Q_OBJECT macro");
@@ -391,6 +395,10 @@ inline T qobject_cast(QObject *object)
 template <class T>
 inline T qobject_cast(const QObject *object)
 {
+    static_assert(std::is_pointer_v<T>,
+                  "qobject_cast requires to cast towards a pointer type");
+    static_assert(std::is_const_v<std::remove_pointer_t<T>>,
+                  "qobject_cast cannot cast away constness (use const_cast)");
     typedef typename std::remove_cv<typename std::remove_pointer<T>::type>::type ObjType;
     static_assert(QtPrivate::HasQ_OBJECT_Macro<ObjType>::Value,
                       "qobject_cast requires the type to have a Q_OBJECT macro");
@@ -452,6 +460,7 @@ public:
 
     inline void reblock() noexcept;
     inline void unblock() noexcept;
+    inline void dismiss() noexcept;
 
 private:
     Q_DISABLE_COPY(QSignalBlocker)
@@ -516,6 +525,11 @@ void QSignalBlocker::unblock() noexcept
     m_inhibited = true;
 }
 
+void QSignalBlocker::dismiss() noexcept
+{
+    m_o = nullptr;
+}
+
 namespace QtPrivate {
     inline QObject & deref_for_methodcall(QObject &o) { return  o; }
     inline QObject & deref_for_methodcall(QObject *o) { return *o; }
