@@ -149,6 +149,7 @@ public:
                               QSize *blockDim) const;
     void textureFormatInfo(QRhiTexture::Format format, const QSize &size,
                            quint32 *bpl, quint32 *byteSize, quint32 *bytesPerPixel) const;
+    bool isStencilSupportingFormat(QRhiTexture::Format format) const;
 
     void registerResource(QRhiResource *res, bool ownsNativeResources = true)
     {
@@ -178,6 +179,16 @@ public:
         cleanupCallbacks.append(callback);
     }
 
+    void addCleanupCallback(const void *key, const QRhi::CleanupCallback &callback)
+    {
+        keyedCleanupCallbacks[key] = callback;
+    }
+
+    void removeCleanupCallback(const void *key)
+    {
+        keyedCleanupCallbacks.remove(key);
+    }
+
     bool sanityCheckGraphicsPipeline(QRhiGraphicsPipeline *ps);
     bool sanityCheckShaderResourceBindings(QRhiShaderResourceBindings *srb);
     void updateLayoutDesc(QRhiShaderResourceBindings *srb);
@@ -238,6 +249,7 @@ private:
     QHash<QRhiResource *, bool> resources;
     QSet<QRhiResource *> pendingDeleteResources;
     QVarLengthArray<QRhi::CleanupCallback, 4> cleanupCallbacks;
+    QHash<const void *, QRhi::CleanupCallback> keyedCleanupCallbacks;
     QElapsedTimer pipelineCreationTimer;
     qint64 accumulatedPipelineCreationTime = 0;
 
