@@ -88,9 +88,7 @@ namespace QTestPrivate {
     \c TestedClass. This instance is used to test for binding loops. By default,
     the method returns a default-constructed \c TestedClass. A custom
     \a helperConstructor should be provided if \c TestedClass is not
-    default-constructible. Some very specific properties cannot be tested for
-    binding loops. Pass a lambda that returns an \c {std::nullptr} as
-    \a helperConstructor in such case.
+    default-constructible. (NOTE: The parameter is currently unused!)
 
     \note Any test calling this method will need to call
     \code
@@ -110,7 +108,12 @@ void testReadWritePropertyBasics(
         std::function<char *(const PropertyType &)> represent =
                 [](const PropertyType &val) { return QTest::toString(val); },
         std::function<std::unique_ptr<TestedClass>(void)> helperConstructor =
-                []() { return std::make_unique<TestedClass>(); })
+                []() {
+                    if constexpr (std::is_default_constructible_v<TestedClass>)
+                        return std::make_unique<TestedClass>();
+                    else
+                        return std::unique_ptr<TestedClass>();
+                })
 {
     // get the property
     const QMetaObject *metaObject = instance.metaObject();
@@ -200,23 +203,7 @@ void testReadWritePropertyBasics(
     if (spy)
         QCOMPARE(spy->size(), 4);
 
-    // test binding loop
-    if (std::unique_ptr<TestedClass> helperObj = std::move(helperConstructor())) {
-        // Reset to 'initial', so that the binding loop test could check the
-        // 'changed' value, because some tests already rely on the 'instance' to
-        // have the 'changed' value once this test passes
-        testedObj.setProperty(propertyName, QVariant::fromValue(initial));
-        const QPropertyBinding<PropertyType> binding([&]() {
-            QObject *obj = static_cast<QObject *>(helperObj.get());
-            obj->setProperty(propertyName, QVariant::fromValue(changed));
-            return obj->property(propertyName).template value<PropertyType>();
-        }, {});
-        bindable.setBinding(binding);
-        QPROPERTY_TEST_COMPARISON_HELPER(
-                testedObj.property(propertyName).template value<PropertyType>(), changed,
-                comparator, represent);
-        QVERIFY2(!binding.error().hasError(), qPrintable(binding.error().description()));
-    }
+    Q_UNUSED(helperConstructor);
 }
 
 /*!
@@ -277,9 +264,7 @@ void testReadWritePropertyBasics(
     \c TestedClass. This instance is used to test for binding loops. By default,
     the method returns a default-constructed \c TestedClass. A custom
     \a helperConstructor should be provided if \c TestedClass is not
-    default-constructible. Some very specific properties cannot be tested for
-    binding loops. Pass a lambda that returns an \c {std::nullptr} as
-    \a helperConstructor in such case.
+    default-constructible. (NOTE: The parameter is currently unused!)
 
     \note Any test calling this method will need to call
     \code
@@ -301,8 +286,15 @@ void testWriteOncePropertyBasics(
         std::function<char *(const PropertyType &)> represent =
                 [](const PropertyType &val) { return QTest::toString(val); },
         std::function<std::unique_ptr<TestedClass>(void)> helperConstructor =
-                []() { return std::make_unique<TestedClass>(); })
+                []() {
+                    if constexpr (std::is_default_constructible_v<TestedClass>)
+                        return std::make_unique<TestedClass>();
+                    else
+                        return std::unique_ptr<TestedClass>();
+                })
 {
+    Q_UNUSED(helperConstructor);
+
     // get the property
     const QMetaObject *metaObject = instance.metaObject();
     QMetaProperty metaProperty = metaObject->property(metaObject->indexOfProperty(propertyName));
@@ -335,19 +327,10 @@ void testWriteOncePropertyBasics(
     propObserver.setBinding(bindable.makeBinding());
     QPROPERTY_TEST_COMPARISON_HELPER(propObserver.value(), prior, comparator, represent);
 
-    // Create a binding that sets the 'changed' value to the property.
-    // This also tests binding loops.
+    // Create a binding that sets the 'changed' value to the property
+    QProperty<PropertyType> propSetter(changed);
     QVERIFY(!bindable.hasBinding());
-    std::unique_ptr<TestedClass> helperObj(std::move(helperConstructor()));
-    QProperty<PropertyType> propSetter(changed); // if the helperConstructor() returns nullptr
-    const QPropertyBinding<PropertyType> binding = helperObj
-            ? Qt::makePropertyBinding([&]() {
-                  QObject *obj = static_cast<QObject *>(helperObj.get());
-                  obj->setProperty(propertyName, QVariant::fromValue(changed));
-                  return obj->property(propertyName).template value<PropertyType>();
-              })
-            : Qt::makePropertyBinding(propSetter);
-    bindable.setBinding(binding);
+    bindable.setBinding(Qt::makePropertyBinding(propSetter));
     QVERIFY(bindable.hasBinding());
 
     QPROPERTY_TEST_COMPARISON_HELPER(
