@@ -4,7 +4,6 @@
 #include "utils.h"
 #include "qmlutils.h"
 #include "qtmoduleinfo.h"
-#include "qtplugininfo.h"
 
 #include <QtCore/QCommandLineOption>
 #include <QtCore/QCommandLineParser>
@@ -115,21 +114,6 @@ static QByteArray formatQtModules(const ModuleBitset &mask, bool option = false)
     return result;
 }
 
-static QString formatQtPlugins(const PluginInformation &pluginInfo)
-{
-    QString result(u'\n');
-    for (const auto &pair : pluginInfo.typeMap()) {
-        result += pair.first;
-        result += u": \n";
-        for (const QString &plugin : pair.second) {
-            result += u"    ";
-            result += plugin;
-            result += u'\n';
-        }
-    }
-    return result;
-}
-
 static Platform platformFromMkSpec(const QString &xSpec)
 {
     if (xSpec.startsWith("win32-"_L1)) {
@@ -178,9 +162,10 @@ struct Options {
     bool quickImports = true;
     bool translations = true;
     bool systemD3dCompiler = true;
+    bool systemDxc = true;
     bool compilerRunTime = false;
+    QStringList disabledPluginTypes;
     bool softwareRasterizer = true;
-    PluginLists pluginSelections;
     Platform platform = WindowsDesktopMsvc;
     ModuleBitset additionalLibraries;
     ModuleBitset disabledLibraries;
@@ -228,8 +213,7 @@ static QString msgFileDoesNotExist(const QString & file)
 
 enum CommandLineParseFlag {
     CommandLineParseError = 0x1,
-    CommandLineParseHelpRequested = 0x2,
-    CommandLineVersionRequested = 0x4
+    CommandLineParseHelpRequested = 0x2
 };
 
 static QCommandLineOption createQMakeOption()
@@ -283,7 +267,7 @@ static int parseEarlyArguments(const QStringList &arguments, Options *options,
     }
 
     if (parser.isSet(qmakeOption) && optVerboseLevel >= 1)
-        std::wcerr << "Warning: -qmake option is deprecated. Use -qtpaths instead.\n";
+        std::wcerr << "Warning: -qmake option is deprecated. Use -qpaths instead.\n";
 
     if (parser.isSet(qtpathsOption) || parser.isSet(qmakeOption)) {
         const QString qtpathsArg = parser.isSet(qtpathsOption) ? parser.value(qtpathsOption)
@@ -330,7 +314,7 @@ static inline int parseArguments(const QStringList &arguments, QCommandLineParse
         "installation (e.g. <QT_DIR\\bin>) to the PATH variable and then run:\n  windeployqt <path-to-app-binary>\n\n"
         "If your application uses Qt Quick, run:\n  windeployqt --qmldir <path-to-app-qml-files> <path-to-app-binary>"_s);
     const QCommandLineOption helpOption = parser->addHelpOption();
-    const QCommandLineOption versionOption = parser->addVersionOption();
+    parser->addVersionOption();
 
     QCommandLineOption dirOption(QStringLiteral("dir"),
                                  QStringLiteral("Use directory instead of binary directory."),
@@ -402,21 +386,6 @@ static inline int parseArguments(const QStringList &arguments, QCommandLineParse
                                          QStringLiteral("plugin types"));
     parser->addOption(skipPluginTypesOption);
 
-    QCommandLineOption addPluginTypesOption(QStringLiteral("add-plugin-types"),
-                                            QStringLiteral("A comma-separated list of plugin types that will be added to deployment (imageformats,iconengines)"),
-                                            QStringLiteral("plugin types"));
-    parser->addOption(addPluginTypesOption);
-
-    QCommandLineOption includePluginsOption(QStringLiteral("include-plugins"),
-                                            QStringLiteral("A comma-separated list of individual plugins that will be added to deployment (scene2d,qjpeg)"),
-                                            QStringLiteral("plugins"));
-    parser->addOption(includePluginsOption);
-
-    QCommandLineOption excludePluginsOption(QStringLiteral("exclude-plugins"),
-                                            QStringLiteral("A comma-separated list of individual plugins that will not be deployed (qsvg,qpdf)"),
-                                            QStringLiteral("plugins"));
-    parser->addOption(excludePluginsOption);
-
     QCommandLineOption noLibraryOption(QStringLiteral("no-libraries"),
                                        QStringLiteral("Skip library deployment."));
     parser->addOption(noLibraryOption);
@@ -449,6 +418,10 @@ static inline int parseArguments(const QStringList &arguments, QCommandLineParse
                                                  QStringLiteral("Skip deployment of the system D3D compiler."));
     parser->addOption(noSystemD3DCompilerOption);
 
+    QCommandLineOption noSystemDxcOption(QStringLiteral("no-system-dxc-compiler"),
+                                         QStringLiteral("Skip deployment of the system DXC (dxcompiler.dll, dxil.dll)."));
+    parser->addOption(noSystemDxcOption);
+
 
     QCommandLineOption compilerRunTimeOption(QStringLiteral("compiler-runtime"),
                                              QStringLiteral("Deploy compiler runtime (Desktop only)."));
@@ -516,8 +489,6 @@ static inline int parseArguments(const QStringList &arguments, QCommandLineParse
     const bool success = parser->parse(arguments);
     if (parser->isSet(helpOption))
         return CommandLineParseHelpRequested;
-    if (parser->isSet(versionOption))
-        return CommandLineVersionRequested;
     if (!success) {
         *errorMessage = parser->errorText();
         return CommandLineParseError;
@@ -533,6 +504,7 @@ static inline int parseArguments(const QStringList &arguments, QCommandLineParse
     if (parser->isSet(translationOption))
         options->languages = parser->value(translationOption).split(u',');
     options->systemD3dCompiler = !parser->isSet(noSystemD3DCompilerOption);
+    options->systemDxc = !parser->isSet(noSystemDxcOption);
     options->quickImports = !parser->isSet(noQuickImportOption);
 
     // default to deployment of compiler runtime for windows desktop configurations
@@ -548,16 +520,7 @@ static inline int parseArguments(const QStringList &arguments, QCommandLineParse
     }
 
     if (parser->isSet(skipPluginTypesOption))
-        options->pluginSelections.disabledPluginTypes = parser->value(skipPluginTypesOption).split(u',');
-
-    if (parser->isSet(addPluginTypesOption))
-        options->pluginSelections.enabledPluginTypes = parser->value(addPluginTypesOption).split(u',');
-
-    if (parser->isSet(includePluginsOption))
-        options->pluginSelections.includedPlugins = parser->value(includePluginsOption).split(u',');
-
-    if (parser->isSet(excludePluginsOption))
-        options->pluginSelections.excludedPlugins = parser->value(excludePluginsOption).split(u',');
+        options->disabledPluginTypes = parser->value(skipPluginTypesOption).split(u',');
 
     if (parser->isSet(releaseWithDebugInfoOption))
         std::wcerr << "Warning: " << releaseWithDebugInfoOption.names().first() << " is obsolete.";
@@ -710,7 +673,7 @@ static inline QString lineBreak(QString s)
     return s;
 }
 
-static inline QString helpText(const QCommandLineParser &p, const PluginInformation &pluginInfo)
+static inline QString helpText(const QCommandLineParser &p)
 {
     QString result = p.helpText();
     // Replace the default-generated text which is too long by a short summary
@@ -729,18 +692,7 @@ static inline QString helpText(const QCommandLineParser &p, const PluginInformat
         "the name prepended by --no- (--no-xml). Available libraries:\n"_L1;
     ModuleBitset mask;
     moduleHelp += lineBreak(QString::fromLatin1(formatQtModules(mask.set(), true)));
-    moduleHelp += u"\n\n";
-    moduleHelp +=
-            u"Qt plugins can be included or excluded individually or by type.\n"
-            u"To deploy or block plugins individually, use the --include-plugins\n"
-            u"and --exclude-plugins options (--include-plugins qjpeg,qsvgicon)\n"
-            u"You can also use the --skip-plugin-types or --add-plugin-types to\n"
-            u"achieve similar results with entire plugin groups, like imageformats, e.g.\n"
-            u"(--add-plugin-types imageformats,iconengines). Exclusion always takes\n"
-            u"precedence over inclusion, and types take precedence over specific plugins.\n"
-            u"For example, including qjpeg, but skipping imageformats, will NOT deploy qjpeg.\n"
-            u"\nDetected available plugins:\n";
-    moduleHelp += formatQtPlugins(pluginInfo);
+    moduleHelp += u'\n';
     result.replace(moduleStart, argumentsStart - moduleStart, moduleHelp);
     return result;
 }
@@ -830,19 +782,13 @@ public:
         SkipSources = 0x2
     };
 
-    explicit QmlDirectoryFileEntryFunction(
-        const QString &moduleSourcePath, Platform platform, DebugMatchMode debugMatchMode, unsigned flags)
+    explicit QmlDirectoryFileEntryFunction(Platform platform, DebugMatchMode debugMatchMode, unsigned flags)
         : m_flags(flags), m_qmlNameFilter(QmlDirectoryFileEntryFunction::qmlNameFilters(flags))
-        , m_dllFilter(platform, debugMatchMode), m_moduleSourcePath(moduleSourcePath)
+        , m_dllFilter(platform, debugMatchMode)
     {}
 
     QStringList operator()(const QDir &dir) const
     {
-        if (moduleSourceDir(dir).canonicalPath() != m_moduleSourcePath) {
-            // If we're in a different module, return nothing.
-            return {};
-        }
-
         QStringList result;
         const QStringList &libraries = m_dllFilter(dir);
         for (const QString &library : libraries) {
@@ -858,17 +804,6 @@ public:
     }
 
 private:
-    static QDir moduleSourceDir(const QDir &dir)
-    {
-        QDir moduleSourceDir = dir;
-        while (!moduleSourceDir.exists(QStringLiteral("qmldir"))) {
-            if (!moduleSourceDir.cdUp()) {
-                return {};
-            }
-        }
-        return moduleSourceDir;
-    }
-
     static inline QStringList qmlNameFilters(unsigned flags)
     {
         QStringList result;
@@ -885,7 +820,6 @@ private:
     const unsigned m_flags;
     NameFilterFileEntryFunction m_qmlNameFilter;
     DllDirectoryFileEntryFunction m_dllFilter;
-    QString m_moduleSourcePath;
 };
 
 static qint64 qtModule(QString module, const QString &infix)
@@ -914,16 +848,15 @@ static qint64 qtModule(QString module, const QString &infix)
 }
 
 // Return the path if a plugin is to be deployed
-static QString deployPlugin(const QString &plugin, const QDir &subDir, const bool dueToModule,
-                            const DebugMatchMode &debugMatchMode, ModuleBitset *usedQtModules,
-                            const ModuleBitset &disabledQtModules,
-                            const PluginLists &pluginSelections, const QString &libraryLocation,
-                            const QString &infix, Platform platform,
-                            bool deployInsightTrackerPlugin)
+static QString deployPlugin(const QString &plugin, const QDir &subDir,
+                            ModuleBitset *usedQtModules, const ModuleBitset &disabledQtModules,
+                            const QStringList &disabledPluginTypes,
+                            const QString &libraryLocation, const QString &infix,
+                            Platform platform, bool deployInsightTrackerPlugin)
 {
     const QString subDirName = subDir.dirName();
     // Filter out disabled plugins
-    if (pluginSelections.disabledPluginTypes.contains(subDirName)) {
+    if (disabledPluginTypes.contains(subDirName)) {
         std::wcout << "Skipping plugin " << plugin << " due to skipped plugin type " << subDirName << '\n';
         return {};
     }
@@ -934,36 +867,7 @@ static QString deployPlugin(const QString &plugin, const QDir &subDir, const boo
         return {};
     }
 
-    const int dotIndex = plugin.lastIndexOf(u'.');
-    // Strip the .dll from the name, and an additional 'd' if it's a debug library with the 'd'
-    // suffix
-    const int stripIndex = debugMatchMode == MatchDebug && platformHasDebugSuffix(platform)
-            ? dotIndex - 1
-            : dotIndex;
-    const QString pluginName = plugin.first(stripIndex);
-
-    if (pluginSelections.excludedPlugins.contains(pluginName)) {
-        std::wcout << "Skipping plugin " << plugin << " due to exclusion option" << '\n';
-        return {};
-    }
-
-    // By default, only deploy qwindows.dll
-    if (subDirName == u"platforms"
-        && !(pluginSelections.includedPlugins.contains(pluginName)
-             || (pluginSelections.enabledPluginTypes.contains(subDirName)))
-        && !pluginName.startsWith(u"qwindows")) {
-        return {};
-    }
-
     const QString pluginPath = subDir.absoluteFilePath(plugin);
-
-    // If dueToModule is false, check if the user included the plugin or the entire type. In the
-    // former's case, only deploy said plugin and not all plugins of that type.
-    const bool requiresPlugin = pluginSelections.includedPlugins.contains(pluginName)
-            || pluginSelections.enabledPluginTypes.contains(subDirName);
-    if (!dueToModule && !requiresPlugin)
-        return {};
-
     // Deploy QUiTools plugins as is without further dependency checking.
     // The user needs to ensure all required libraries are present (would
     // otherwise pull QtWebEngine for its plugin).
@@ -1001,28 +905,18 @@ static QString deployPlugin(const QString &plugin, const QDir &subDir, const boo
         *usedQtModules |= missingModules;
         if (optVerboseLevel) {
             std::wcout << "Adding " << formatQtModules(missingModules).constData()
-                << " for " << plugin << " from plugin type: " << subDirName << '\n';
+                << " for " << plugin << '\n';
         }
     }
     return pluginPath;
 }
 
-static bool needsPluginType(const QString &subDirName, const PluginInformation &pluginInfo,
-                            const PluginLists &pluginSelections)
-{
-    bool needsTypeForPlugin = false;
-    for (const QString &plugin: pluginSelections.includedPlugins) {
-        if (pluginInfo.isTypeForPlugin(subDirName, plugin))
-            needsTypeForPlugin = true;
-    }
-    return (pluginSelections.enabledPluginTypes.contains(subDirName) || needsTypeForPlugin);
-}
-
 QStringList findQtPlugins(ModuleBitset *usedQtModules, const ModuleBitset &disabledQtModules,
-                          const PluginInformation &pluginInfo, const PluginLists &pluginSelections,
+                          const QStringList &disabledPluginTypes,
                           const QString &qtPluginsDirName, const QString &libraryLocation,
-                          const QString &infix, DebugMatchMode debugMatchModeIn, Platform platform,
-                          QString *platformPlugin, bool deployInsightTrackerPlugin)
+                          const QString &infix,
+                          DebugMatchMode debugMatchModeIn, Platform platform, QString *platformPlugin,
+                          bool deployInsightTrackerPlugin)
 {
     if (qtPluginsDirName.isEmpty())
         return QStringList();
@@ -1038,24 +932,30 @@ QStringList findQtPlugins(ModuleBitset *usedQtModules, const ModuleBitset &disab
             }
             continue;
         }
-        const bool dueToModule = usedQtModules->test(module);
-        if (dueToModule || needsPluginType(subDirName, pluginInfo, pluginSelections)) {
+        if (usedQtModules->test(module)) {
             const DebugMatchMode debugMatchMode = (module == QtWebEngineCoreModuleId)
                 ? MatchDebugOrRelease // QTBUG-44331: Debug detection does not work for webengine, deploy all.
                 : debugMatchModeIn;
             QDir subDir(subDirFi.absoluteFilePath());
-            std::wcout << "Adding in plugin type " << subDirFi.baseName() << " for module: " << qtModuleEntries.moduleById(module).name << '\n';
 
+            // Filter for platform or any.
+            QString filter;
             const bool isPlatformPlugin = subDirName == "platforms"_L1;
-            const QStringList plugins =
-                    findSharedLibraries(subDir, platform, debugMatchMode, QString());
+            if (isPlatformPlugin) {
+                filter = QStringLiteral("qwindows");
+                if (!infix.isEmpty())
+                    filter += infix;
+            } else {
+                filter = u"*"_s;
+            }
+            const QStringList plugins = findSharedLibraries(subDir, platform, debugMatchMode, filter);
             for (const QString &plugin : plugins) {
                 const QString pluginPath =
-                        deployPlugin(plugin, subDir, dueToModule, debugMatchMode, usedQtModules,
-                                     disabledQtModules, pluginSelections, libraryLocation, infix,
-                                     platform, deployInsightTrackerPlugin);
+                    deployPlugin(plugin, subDir, usedQtModules, disabledQtModules,
+                                 disabledPluginTypes, libraryLocation, infix, platform,
+                                 deployInsightTrackerPlugin);
                 if (!pluginPath.isEmpty()) {
-                    if (isPlatformPlugin && plugin.startsWith(u"qwindows"))
+                    if (isPlatformPlugin)
                         *platformPlugin = subDir.absoluteFilePath(plugin);
                     result.append(pluginPath);
                 }
@@ -1326,7 +1226,7 @@ static QString getIcuVersion(const QString &libName)
 }
 
 static DeployResult deploy(const Options &options, const QMap<QString, QString> &qtpathsVariables,
-                           const PluginInformation &pluginInfo, QString *errorMessage)
+                           QString *errorMessage)
 {
     DeployResult result;
 
@@ -1511,8 +1411,8 @@ static DeployResult deploy(const Options &options, const QMap<QString, QString>
             &result.deployedQtLibraries,
             // For non-QML applications, disable QML to prevent it from being pulled in by the
             // qtaccessiblequick plugin.
-            disabled, pluginInfo,
-            options.pluginSelections, qtpathsVariables.value(QStringLiteral("QT_INSTALL_PLUGINS")),
+            disabled,
+            options.disabledPluginTypes, qtpathsVariables.value(QStringLiteral("QT_INSTALL_PLUGINS")),
             libraryLocation, infix, debugMatchMode, options.platform, &platformPlugin,
             options.deployInsightTrackerPlugin);
 
@@ -1558,6 +1458,13 @@ static DeployResult deploy(const Options &options, const QMap<QString, QString>
                 deployedQtLibraries.push_back(d3dCompiler);
             }
         }
+        if (options.systemDxc) {
+            const QStringList dxcLibs = findDxc(options.platform, qtBinDir, wordSize);
+            if (!dxcLibs.isEmpty())
+                deployedQtLibraries.append(dxcLibs);
+            else
+                std::wcerr << "Warning: Cannot find any version of the dxcompiler.dll and dxil.dll.\n";
+        }
     } // Windows
 
     // Update libraries
@@ -1632,8 +1539,7 @@ static DeployResult deploy(const Options &options, const QMap<QString, QString>
             unsigned qmlDirectoryFileFlags = 0;
             if (options.deployPdb)
                 qmlDirectoryFileFlags |= QmlDirectoryFileEntryFunction::DeployPdb;
-            if (!updateFile(module.sourcePath, QmlDirectoryFileEntryFunction(module.sourcePath,
-                                                                             options.platform,
+            if (!updateFile(module.sourcePath, QmlDirectoryFileEntryFunction(options.platform,
                                                                              debugMatchMode,
                                                                              qmlDirectoryFileFlags),
                             installPath, updateFileFlags, options.json, errorMessage)) {
@@ -1657,8 +1563,7 @@ static DeployResult deploy(const Options &options, const QMap<QString, QString>
 }
 
 static bool deployWebProcess(const QMap<QString, QString> &qtpathsVariables, const char *binaryName,
-                             const PluginInformation &pluginInfo, const Options &sourceOptions,
-                             QString *errorMessage)
+                             const Options &sourceOptions, QString *errorMessage)
 {
     // Copy the web process and its dependencies
     const QString webProcess = webProcessBinary(binaryName, sourceOptions.platform);
@@ -1670,12 +1575,11 @@ static bool deployWebProcess(const QMap<QString, QString> &qtpathsVariables, con
     options.binaries.append(options.directory + u'/' + webProcess);
     options.quickImports = false;
     options.translations = false;
-    return deploy(options, qtpathsVariables, pluginInfo, errorMessage);
+    return deploy(options, qtpathsVariables, errorMessage);
 }
 
 static bool deployWebEngineCore(const QMap<QString, QString> &qtpathsVariables,
-                                const PluginInformation &pluginInfo, const Options &options,
-                                bool isDebug, QString *errorMessage)
+                                const Options &options, bool isDebug, QString *errorMessage)
 {
     static const char *installDataFiles[] = { "icudtl.dat",
                                               "qtwebengine_devtools_resources.pak",
@@ -1689,7 +1593,7 @@ static bool deployWebEngineCore(const QMap<QString, QString> &qtpathsVariables,
         webEngineProcessName.append('d');
     if (optVerboseLevel)
         std::wcout << "Deploying: " << webEngineProcessName.constData() << "...\n";
-    if (!deployWebProcess(qtpathsVariables, webEngineProcessName, pluginInfo, options, errorMessage))
+    if (!deployWebProcess(qtpathsVariables, webEngineProcessName, options, errorMessage))
         return false;
     const QString resourcesSubDir = QStringLiteral("/resources");
     const QString resourcesSourceDir = qtpathsVariables.value(QStringLiteral("QT_INSTALL_DATA"))
@@ -1764,18 +1668,18 @@ int main(int argc, char **argv)
     const QMap<QString, QString> qtpathsVariables =
             queryQtPaths(options.qtpathsBinary, &errorMessage);
     const QString xSpec = qtpathsVariables.value(QStringLiteral("QMAKE_XSPEC"));
-    if (qtpathsVariables.isEmpty() || xSpec.isEmpty()
-        || !qtpathsVariables.contains(QStringLiteral("QT_INSTALL_BINS"))) {
-        std::wcerr << "Unable to query qtpaths: " << errorMessage << '\n';
-        return 1;
-    }
-
     options.platform = platformFromMkSpec(xSpec);
     if (options.platform == UnknownPlatform) {
         std::wcerr << "Unsupported platform " << xSpec << '\n';
         return 1;
     }
 
+    if (qtpathsVariables.isEmpty() || xSpec.isEmpty()
+        || !qtpathsVariables.contains(QStringLiteral("QT_INSTALL_BINS"))) {
+        std::wcerr << "Unable to query qtpaths: " << errorMessage << '\n';
+        return 1;
+    }
+
     // Read the Qt module information from the Qt installation directory.
     const QString modulesDir
             = qtpathsVariables.value(QLatin1String("QT_INSTALL_ARCHDATA"))
@@ -1789,10 +1693,6 @@ int main(int argc, char **argv)
     }
     assignKnownModuleIds();
 
-    // Read the Qt plugin types information from the Qt installation directory.
-    PluginInformation pluginInfo{};
-    pluginInfo.generateAvailablePlugins(qtpathsVariables, options.platform);
-
     // Parse the full command line.
     {
         QCommandLineParser parser;
@@ -1800,12 +1700,8 @@ int main(int argc, char **argv)
         const int result = parseArguments(QCoreApplication::arguments(), &parser, &options, &errorMessage);
         if (result & CommandLineParseError)
             std::wcerr << errorMessage << "\n\n";
-        if (result & CommandLineVersionRequested) {
-            std::fputs(QT_VERSION_STR "\n", stdout);
-            return 0;
-        }
         if (result & CommandLineParseHelpRequested)
-            std::fputs(qPrintable(helpText(parser, pluginInfo)), stdout);
+            std::fputs(qPrintable(helpText(parser)), stdout);
         if (result & CommandLineParseError)
             return 1;
         if (result & CommandLineParseHelpRequested)
@@ -1823,15 +1719,14 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    const DeployResult result = deploy(options, qtpathsVariables, pluginInfo, &errorMessage);
+    const DeployResult result = deploy(options, qtpathsVariables, &errorMessage);
     if (!result) {
         std::wcerr << errorMessage << '\n';
         return 1;
     }
 
     if (result.deployedQtLibraries.test(QtWebEngineCoreModuleId)) {
-        if (!deployWebEngineCore(qtpathsVariables, pluginInfo, options, result.isDebug,
-                                 &errorMessage)) {
+        if (!deployWebEngineCore(qtpathsVariables, options, result.isDebug, &errorMessage)) {
             std::wcerr << errorMessage << '\n';
             return 1;
         }
