@@ -1,6 +1,7 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
+#undef QT_NO_FOREACH // this file contains unported legacy Q_FOREACH uses
 
 #include <QTest>
 #include <QtTest/qtesttouch.h>
@@ -348,21 +349,34 @@ void tst_Gestures::cleanup()
     QGestureRecognizer::unregisterRecognizer(CustomGesture::GestureType);
 }
 
+static QtMessageHandler originalMessageHandler = nullptr;
+static bool skipCustomGesture = false;
+static void messageHandler(QtMsgType type, const QMessageLogContext &context, const QString &text)
+{
+    if (type == QtMsgType::QtWarningMsg && text == "QGestureManager::deliverEvent: could not find the target for gesture")
+        skipCustomGesture = true;
+    originalMessageHandler(type, context, text);
+}
+
 void tst_Gestures::customGesture()
 {
+    originalMessageHandler = qInstallMessageHandler(messageHandler);
+    auto guard = qScopeGuard([](){ qInstallMessageHandler(originalMessageHandler); });
     GestureWidget widget;
     widget.grabGesture(CustomGesture::GestureType, Qt::DontStartGestureOnChildren);
     widget.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&widget));
+    QVERIFY(QTest::qWaitForWindowActive(&widget));
 
     CustomEvent event;
     event.hotSpot = widget.mapToGlobal(QPoint(5,5));
     event.hasHotSpot = true;
     sendCustomGesture(&event, &widget);
+    if (skipCustomGesture)
+        QSKIP("QApplication unable to access test widget.");
 
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
     static const int TotalCustomEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialMaybeThreshold + 1;
-    QCOMPARE(widget.customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(widget.customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(widget.gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(widget.gestureOverrideEventsReceived, 0);
     QCOMPARE(widget.events.all.size(), TotalGestureEventsCount);
@@ -422,7 +436,7 @@ void tst_Gestures::gestureOverChild()
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
     static const int TotalCustomEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialMaybeThreshold + 1;
 
-    QCOMPARE(child->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(child->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(widget.customEventsReceived, 0);
     QCOMPARE(child->gestureEventsReceived, 0);
     QCOMPARE(child->gestureOverrideEventsReceived, 0);
@@ -437,7 +451,7 @@ void tst_Gestures::gestureOverChild()
 
     sendCustomGesture(&event, child);
 
-    QCOMPARE(child->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(child->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(widget.customEventsReceived, 0);
 
     QCOMPARE(child->gestureEventsReceived, 0);
@@ -469,7 +483,7 @@ void tst_Gestures::multipleWidgetOnlyGestureInTree()
     CustomEvent event;
     sendCustomGesture(&event, child);
 
-    QCOMPARE(child->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(child->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(parent.customEventsReceived, 0);
     QCOMPARE(child->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(child->gestureOverrideEventsReceived, 0);
@@ -482,8 +496,8 @@ void tst_Gestures::multipleWidgetOnlyGestureInTree()
     // same for the parent widget
     sendCustomGesture(&event, &parent);
 
-    QCOMPARE(child->customEventsReceived, 0);
     QCOMPARE(parent.customEventsReceived, TotalCustomEventsCount);
+    QCOMPARE(child->customEventsReceived, 0);
     QCOMPARE(child->gestureEventsReceived, 0);
     QCOMPARE(child->gestureOverrideEventsReceived, 0);
     QCOMPARE(parent.gestureEventsReceived, TotalGestureEventsCount);
@@ -510,7 +524,7 @@ void tst_Gestures::conflictingGestures()
     CustomEvent event;
     sendCustomGesture(&event, child);
 
-    QCOMPARE(child->gestureOverrideEventsReceived, 1);
+    QTRY_COMPARE(child->gestureOverrideEventsReceived, 1);
     QCOMPARE(child->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(parent.gestureOverrideEventsReceived, 0);
     QCOMPARE(parent.gestureEventsReceived, 0);
@@ -578,8 +592,8 @@ void tst_Gestures::conflictingGestures()
     // sending events to the child and making sure there is no conflict
     sendCustomGesture(&event, child);
 
+    QTRY_COMPARE(child->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(child->gestureOverrideEventsReceived, 1);
-    QCOMPARE(child->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(parent.gestureOverrideEventsReceived, 1);
     QCOMPARE(parent.gestureEventsReceived, TotalGestureEventsCount);
 
@@ -597,8 +611,8 @@ void tst_Gestures::conflictingGestures()
     child->acceptGestureOverride = true;
     sendCustomGesture(&event, child);
 
+    QTRY_VERIFY(child->gestureEventsReceived > TotalGestureEventsCount);
     QCOMPARE(child->gestureOverrideEventsReceived, 1);
-    QVERIFY(child->gestureEventsReceived > TotalGestureEventsCount);
     QCOMPARE(child->events.all.size(), TotalGestureEventsCount + ContinuousGestureEventsCount);
     QCOMPARE(parent.gestureOverrideEventsReceived, 0);
     QCOMPARE(parent.gestureEventsReceived, 0);
@@ -614,7 +628,7 @@ void tst_Gestures::finishedWithoutStarted()
     ev.serial = CustomGesture::SerialFinishedThreshold;
     QApplication::sendEvent(&widget, &ev);
 
-    QCOMPARE(widget.customEventsReceived, 1);
+    QTRY_COMPARE(widget.customEventsReceived, 1);
     QCOMPARE(widget.gestureEventsReceived, 2);
     QCOMPARE(widget.gestureOverrideEventsReceived, 0);
     QCOMPARE(widget.events.all.size(), 2);
@@ -636,7 +650,7 @@ void tst_Gestures::unknownGesture()
 
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
 
-    QCOMPARE(widget.gestureEventsReceived, TotalGestureEventsCount);
+    QTRY_COMPARE(widget.gestureEventsReceived, TotalGestureEventsCount);
 }
 
 static const QColor InstanceColors[] = {
@@ -840,7 +854,7 @@ void tst_Gestures::graphicsItemGesture()
     QTest::ignoreMessage(QtWarningMsg, "QGestureManager::deliverEvent: could not find the target for gesture");
     sendCustomGesture(&event, item, &scene);
 
-    QCOMPARE(item->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(item->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(item->gestureEventsReceived, 0);
     QCOMPARE(item->gestureOverrideEventsReceived, 0);
 
@@ -901,7 +915,7 @@ void tst_Gestures::graphicsView()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item, &scene);
 
-    QCOMPARE(item->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(item->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(item->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item->gestureOverrideEventsReceived, 0);
 
@@ -936,8 +950,8 @@ void tst_Gestures::graphicsView()
     item->grabGesture(CustomGesture::GestureType);
     sendCustomGesture(&event, item, &newScene);
 
+    QTRY_COMPARE(item->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item->customEventsReceived, TotalCustomEventsCount);
-    QCOMPARE(item->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item->gestureOverrideEventsReceived, 0);
 }
 
@@ -977,12 +991,12 @@ void tst_Gestures::graphicsItemTreeGesture()
 
     item1->ignoredGestures << CustomGesture::GestureType;
     sendCustomGesture(&event, item1_child1, &scene);
+    QTRY_COMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1_child1->gestureOverrideEventsReceived, 0);
     QCOMPARE(item1_child1->gestureEventsReceived, 0);
     QCOMPARE(item1_child2->gestureEventsReceived, 0);
     QCOMPARE(item1_child2->gestureOverrideEventsReceived, 0);
     QCOMPARE(item1->gestureOverrideEventsReceived, 0);
-    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
 
     item1->reset(); item1_child1->reset(); item1_child2->reset();
 
@@ -991,7 +1005,7 @@ void tst_Gestures::graphicsItemTreeGesture()
     item1->ignoredGestures << CustomGesture::GestureType;
     item1_child1->ignoredGestures << CustomGesture::GestureType;
     sendCustomGesture(&event, item1_child1, &scene);
-    QCOMPARE(item1_child1->gestureOverrideEventsReceived, 1);
+    QTRY_COMPARE(item1_child1->gestureOverrideEventsReceived, 1);
     QCOMPARE(item1_child1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1_child2->gestureEventsReceived, 0);
     QCOMPARE(item1_child2->gestureOverrideEventsReceived, 0);
@@ -1038,9 +1052,9 @@ void tst_Gestures::explicitGraphicsObjectTarget()
 
     sendCustomGesture(&event, item1, &scene);
 
-    QCOMPARE(item1->gestureEventsReceived, 0);
+    QTRY_COMPARE(item2_child1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
-    QCOMPARE(item2_child1->gestureEventsReceived, TotalGestureEventsCount);
+    QCOMPARE(item1->gestureEventsReceived, 0);
     QCOMPARE(item2_child1->gestureOverrideEventsReceived, 1);
     QCOMPARE(item2_child1->events.all.size(), TotalGestureEventsCount);
     for(int i = 0; i < item2_child1->events.all.size(); ++i)
@@ -1081,7 +1095,7 @@ void tst_Gestures::gestureOverChildGraphicsItem()
     item2_child1->setPos(0, 0);
 
     view.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&view));
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(&view));
     view.ensureVisible(scene.sceneRect());
 
     item1->grabGesture(CustomGesture::GestureType);
@@ -1094,7 +1108,7 @@ void tst_Gestures::gestureOverChildGraphicsItem()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item0, &scene);
 
-    QCOMPARE(item0->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(item0->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(item2_child1->gestureEventsReceived, 0);
     QCOMPARE(item2_child1->gestureOverrideEventsReceived, 0);
     QCOMPARE(item2->gestureEventsReceived, 0);
@@ -1110,11 +1124,11 @@ void tst_Gestures::gestureOverChildGraphicsItem()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item0, &scene);
 
-    QCOMPARE(item2_child1->gestureEventsReceived, 0);
-    QCOMPARE(item2_child1->gestureOverrideEventsReceived, 0);
+    QTRY_COMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item2->gestureEventsReceived, 1);
     QCOMPARE(item2->gestureOverrideEventsReceived, 1);
-    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
+    QCOMPARE(item2_child1->gestureEventsReceived, 0);
+    QCOMPARE(item2_child1->gestureOverrideEventsReceived, 0);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
 
     item0->reset(); item1->reset(); item2->reset(); item2_child1->reset();
@@ -1126,9 +1140,9 @@ void tst_Gestures::gestureOverChildGraphicsItem()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item0, &scene);
 
+    QTRY_COMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item2_child1->gestureEventsReceived, 0);
     QCOMPARE(item2_child1->gestureOverrideEventsReceived, 0);
-    QCOMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item2->gestureOverrideEventsReceived, 1);
     QCOMPARE(item1->gestureEventsReceived, 1);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
@@ -1143,9 +1157,9 @@ void tst_Gestures::gestureOverChildGraphicsItem()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item0, &scene);
 
+    QTRY_COMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item2_child1->gestureEventsReceived, 0);
     QCOMPARE(item2_child1->gestureOverrideEventsReceived, 0);
-    QCOMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item2->gestureOverrideEventsReceived, 1);
     QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
@@ -1176,16 +1190,16 @@ void tst_Gestures::twoGesturesOnDifferentLevel()
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
     static const int TotalCustomEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialMaybeThreshold + 1;
 
-    QCOMPARE(child->customEventsReceived, TotalCustomEventsCount);
+    QTRY_COMPARE(child->customEventsReceived, TotalCustomEventsCount);
     QCOMPARE(child->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(child->gestureOverrideEventsReceived, 0);
     QCOMPARE(child->events.all.size(), TotalGestureEventsCount);
     for(int i = 0; i < child->events.all.size(); ++i)
         QCOMPARE(child->events.all.at(i), SecondGesture);
 
+    QCOMPARE(parent.events.all.size(), TotalGestureEventsCount);
     QCOMPARE(parent.gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(parent.gestureOverrideEventsReceived, 0);
-    QCOMPARE(parent.events.all.size(), TotalGestureEventsCount);
     for(int i = 0; i < child->events.all.size(); ++i)
         QCOMPARE(parent.events.all.at(i), CustomGesture::GestureType);
 }
@@ -1229,7 +1243,7 @@ void tst_Gestures::multipleGesturesInTree()
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
 
     // gesture override events
-    QCOMPARE(D->overrideEvents.all.count(FirstGesture), 1);
+    QTRY_COMPARE(D->overrideEvents.all.count(FirstGesture), 1);
     QCOMPARE(D->overrideEvents.all.count(SecondGesture), 0);
     QCOMPARE(D->overrideEvents.all.count(ThirdGesture), 1);
 
@@ -1315,7 +1329,7 @@ void tst_Gestures::multipleGesturesInComplexTree()
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
 
     // gesture override events
-    QCOMPARE(D->overrideEvents.all.count(FirstGesture), 1);
+    QTRY_COMPARE(D->overrideEvents.all.count(FirstGesture), 1);
     QCOMPARE(D->overrideEvents.all.count(SecondGesture), 0);
     QCOMPARE(D->overrideEvents.all.count(ThirdGesture), 1);
 
@@ -1425,7 +1439,7 @@ void tst_Gestures::ungrabGesture() // a method on QWidget
     // sending an event will cause the QGesture objects to be instantiated for the widgets
     sendCustomGesture(&event, b);
 
-    QCOMPARE(a->gestures.size(), 1);
+    QTRY_COMPARE(a->gestures.size(), 1);
     QPointer<QGesture> customGestureA;
     customGestureA = *(a->gestures.begin());
     QVERIFY(!customGestureA.isNull());
@@ -1442,7 +1456,7 @@ void tst_Gestures::ungrabGesture() // a method on QWidget
     // sending an event will cause the QGesture objects to be instantiated for the widget
     sendCustomGesture(&event, a);
 
-    QCOMPARE(a->gestures.size(), 1);
+    QTRY_COMPARE(a->gestures.size(), 1);
     customGestureA = *(a->gestures.begin());
     QVERIFY(!customGestureA.isNull());
     QCOMPARE(customGestureA->gestureType(), CustomGesture::GestureType);
@@ -1450,7 +1464,7 @@ void tst_Gestures::ungrabGesture() // a method on QWidget
 
     a->ungrabGesture(CustomGesture::GestureType);
     //We changed the deletion of Gestures to lazy during QT-4022, so we can't ensure the QGesture is deleted until now
-    QVERIFY(!customGestureB.isNull());
+    QTRY_VERIFY(!customGestureB.isNull());
 
     a->gestures.clear();
     a->reset();
@@ -1646,7 +1660,7 @@ void tst_Gestures::graphicsViewParentPropagation()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item0, &scene);
 
-    QCOMPARE(item1_c1_c1->gestureEventsReceived, TotalGestureEventsCount);
+    QTRY_COMPARE(item1_c1_c1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1_c1_c1->gestureOverrideEventsReceived, 1);
     QCOMPARE(item1_c1->gestureEventsReceived, TotalGestureEventsCount-1);
     QCOMPARE(item1_c1->gestureOverrideEventsReceived, 1);
@@ -1719,11 +1733,11 @@ void tst_Gestures::panelPropagation()
     event.hasHotSpot = true;
     sendCustomGesture(&event, item1, &scene);
 
+    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1_child1_child1->gestureEventsReceived, 0);
     QCOMPARE(item1_child1_child1->gestureOverrideEventsReceived, 0);
     QCOMPARE(item1_child1->gestureEventsReceived, 0);
     QCOMPARE(item1_child1->gestureOverrideEventsReceived, 0);
-    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
     QCOMPARE(item0->gestureEventsReceived, 0);
     QCOMPARE(item0->gestureOverrideEventsReceived, 1);
@@ -1773,7 +1787,7 @@ void tst_Gestures::panelPropagation()
 
     QCOMPARE(item1_child1_child1->gestureEventsReceived, 0);
     QCOMPARE(item1_child1_child1->gestureOverrideEventsReceived, 0);
-    QCOMPARE(item1_child1->gestureEventsReceived, TotalGestureEventsCount);
+    QTRY_COMPARE(item1_child1->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1_child1->gestureOverrideEventsReceived, 0);
     QCOMPARE(item1->gestureEventsReceived, 0);
     QCOMPARE(item1->gestureOverrideEventsReceived, 0);
@@ -1966,7 +1980,7 @@ void tst_Gestures::viewportCoordinates()
     event.hotSpot = mapToGlobal(item1->boundingRect().center(), item1, &view);
     event.hasHotSpot = true;
     sendCustomGesture(&event, item1, &scene);
-    QVERIFY(item1->gestureEventsReceived != 0);
+    QTRY_VERIFY(item1->gestureEventsReceived != 0);
 }
 
 void tst_Gestures::partialGesturePropagation()
@@ -2009,13 +2023,14 @@ void tst_Gestures::partialGesturePropagation()
 
     static const int TotalGestureEventsCount = CustomGesture::SerialFinishedThreshold - CustomGesture::SerialStartedThreshold + 1;
 
+    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
+    QCOMPARE(item2->gestureEventsReceived, TotalGestureEventsCount-2); // except for started and finished
+
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
     QCOMPARE(item2->gestureOverrideEventsReceived, 1);
     QCOMPARE(item3->gestureOverrideEventsReceived, 1);
     QCOMPARE(item4->gestureOverrideEventsReceived, 1);
 
-    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
-    QCOMPARE(item2->gestureEventsReceived, TotalGestureEventsCount-2); // except for started and finished
     QCOMPARE(item3->gestureEventsReceived, 0);
     QCOMPARE(item4->gestureEventsReceived, 0);
 }
@@ -2298,8 +2313,8 @@ void tst_Gestures::conflictingGesturesInGraphicsView()
     event.hotSpot = mapToGlobal(item2->boundingRect().center(), item2, &view);
     event.hasHotSpot = true;
     sendCustomGesture(&event, item2, &scene);
+    QTRY_COMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item2->gestureOverrideEventsReceived, 1);
-    QCOMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
     QCOMPARE(item1->gestureEventsReceived, 0);
 
@@ -2311,8 +2326,8 @@ void tst_Gestures::conflictingGesturesInGraphicsView()
     event.hotSpot = mapToGlobal(item2->boundingRect().center(), item2, &view);
     event.hasHotSpot = true;
     sendCustomGesture(&event, item2, &scene);
-    QCOMPARE(item2->gestureOverrideEventsReceived, 1);
     QCOMPARE(item2->gestureEventsReceived, TotalGestureEventsCount);
+    QCOMPARE(item2->gestureOverrideEventsReceived, 1);
     QCOMPARE(item1->gestureOverrideEventsReceived, 0);
     QCOMPARE(item1->gestureEventsReceived, 0);
 
@@ -2324,11 +2339,11 @@ void tst_Gestures::conflictingGesturesInGraphicsView()
     event.hotSpot = mapToGlobal(item2->boundingRect().center(), item2, &view);
     event.hasHotSpot = true;
     sendCustomGesture(&event, item2, &scene);
+    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
 
     QCOMPARE(item2->gestureOverrideEventsReceived, 1);
     QCOMPARE(item2->gestureEventsReceived, 0);
     QCOMPARE(item1->gestureOverrideEventsReceived, 1);
-    QCOMPARE(item1->gestureEventsReceived, TotalGestureEventsCount);
 }
 
 class NoConsumeWidgetBug13501 :public QWidget
